function [f,x] = amoeba(x0,precx,precf)
%
%  [f,x,iter] = amoeba(x0,precx,precf)
%
% This code is called the "Downhill Simplex Method"
% attributed to Nelder and Mead
% for finding maxima or minima in a single multidimensional
% function.
%
% inputs:
%  x0 is the initial guess, column vector of length n
%  precx is the solution variable precision, scalar
%  precf is the solution function precision, scalar
%  the objective function is entered in the function funkeval in the
%  funkeval.m file
%
% outputs:
% 
% x = converged solution, column vector of length n
% f = optimized value of function, scalar
% iter = number of iterations
% err = RMS value of relative error on x, scalar
%
% example usage:
%
%  [f,x] = amoeba([1; 2],1.0e-6,1.0e-6)
%
%  This code is based on the "amoeba" code of 
%  Press, W.H.,  Flannery, B.P., Teukolsky, S.A., Vetterling, W.T.
%  "Numerical Recipes", Cambridge University Pres, Cambridge, 1986, p. 292-293.
%  
%  Fortran translated to Matlab by David Keffer
%  Department of Chemical Engineering, University of Tennessee
%  original translation date: May,1999
% last modified date: April 5, 2013
%

%  parameters for the "Downhill Simplex Method"
alpha = 1.0;
beta = 0.5;
gamma = 2.0;
%  maximum iterations
maxit = 10000;
%
%  np = number of dimensions
np = max(size(x0));
np1 = np+1;
%
% initialize lambda start-up vector
% lambda controls the size of the initial volume
%
lambda = zeros(np,1);
lambda(1:np) = 0.01;
%
%  initialize p and y
%
%  vector of vertice coordinates
   p = zeros(np1,np);
   for j = 1:1:np1
      for i = 1:1:np
         p(j,i) = x0(i);
      end   
   end
   for j = 1:1:np
      p(j+1,j) = x0(j)*(1.0+lambda(j));
   end
%
%  vector of function evaluations
%
   y = zeros(np1,1);
   for j = 1:1:np1
      y(j) = funkeval(p(j,:));
   end
%
%  print initial information
%
for i = 1:1:np1
   fprintf (1,'i =  %3i', i );
   for j = 1:1:np
      fprintf (1,' %15.7e', p(i,j));
   end   
      fprintf (1,' f = %15.7e \n', y(i));
end
%
%initialize error and iteration count
%
err_f = 100.0;
err_x = 100.0;
iter = 0.0;
%
%
%  start convergence loop
%
while (err_f > precf | err_x > precx)
   %  locate highest value in y-vector
   [maxval1, maxpos1] = max(y);
   y(maxpos1) = 0.0;
   %  locate second highest value in y-vector
   [maxval2, maxpos2] = max(y);
   y(maxpos1) = maxval1;
   %  locate lowest value in y-vector
   [minval1, minpos1] = min(y);
   %
   %  evaluate error
   %
   err_f = 2*abs(y(maxpos1)-y(minpos1))/abs(y(maxpos1)+y(minpos1));
   err_x = 0.0d0;
   for i = 1:1:np
      err_x = err_x + (2*abs(p(maxpos1,i)-p(minpos1,i))/abs(p(maxpos1,i)+p(minpos1,i)))^2;
   end
   err_x = sqrt(err_x/np);
   %
   %  new iteration
   %
   pbar = zeros(np,1);
   for i = 1:np1
      if (i ~= maxpos1)
         for j = 1:1:np
            pbar(j) = pbar(j) + p(i,j);
         end
      end
   end
   pbar = pbar/np;
   %
   %  find coordinates of reflected point
   %
   pr = zeros(np,1);
   prr = zeros(np,1);
   for j = 1:1:np
      pr(j) = (1+alpha)*pbar(j) - alpha*p(maxpos1,j);
   end
   %
   % evaluate function at reflected point
   %
   yr = funkeval(pr);
   %
   %  reflected point is better than anything else
   %
   if ( yr < y(minpos1) )
      %
      %  try an additional extrapolation
      %
      prr = gamma*pr + (1-gamma)*pbar;
      yrr = funkeval(prr);
      if ( yrr < y(minpos1) )
         %  additional extrapolation good
         for j = 1:1:np
         	p(maxpos1,j) = prr(j);
         end
         y(maxpos1) = yrr;
      else
         %  additional extrapolation bad, use reflection
         for j = 1:1:np
         	p(maxpos1,j) = pr(j);
         end
         y(maxpos1) = yr;
      end
   elseif ( yr > y(maxpos2) )
      if (yr < y(maxpos1))
         for j = 1:1:np
         	p(maxpos1,j) = pr(j);
         end
         y(maxpos1) = yr;
      end
      %
      % look for better intermediate point (contraction)
      %
      for j = 1:1:np
         prr(j) = beta*p(maxpos1,j) + (1-beta)*pbar(j);
      end
      yrr = funkeval(prr);
      if ( yrr < y(maxpos1) )
         % contraction is good, accept it
         for j = 1:1:np
         	p(maxpos1,j) = prr(j);
         end
         y(maxpos1) = yrr;
      else
         % contraction is bad, contract instead around best point
         for i = 1:1:np1
            if (i ~= minpos1)
               for j = 1:1:np
               	pr(j) = 0.5*(p(i,j) + p(minpos1,j));
                  p(i,j) = pr(j);
               end
               yr = funkeval(pr);
            end
         end
      end
   else 
      %
      %  reflected point was bad, make it new high value
      %
      for j = 1:1:np
         p(maxpos1,j) = pr(j);
      end
      y(maxpos1) = yr;
   end
   %
   %  check iterations
   %
   iter = iter +1;
   if (iter > maxit)
      for j = 1:1:np
         x(j) = p(minpos1,j);
         fprintf (1,'xp(%3i) = %15.7e \n', j, x(j) );
      end
      f = y(minpos1)
      error ('maximum number of iterations exceeded ');
   end
%   fprintf(' %5i  %15.7e  %15.7e  %15.7e %15.7e  %15.7e %15.7e\n', iter, p(minpos1,1:3), y(minpos1), err_x, err_f);
end
%
% store answer in solution vector
%
for j = 1:1:np
	x(j) = p(minpos1,j);
end
f = y(minpos1);









   