function [f,xmin] = brent(ax,bx,cx,tol,iprint,key)
%
%  This routine is called the  Brent Method"
%  for finding a minimum in one dimension.
%
%  THIS CODE ONLY FINDS MINIMA, NOT MAXIMA.
%
%  This code is based on the "brent" code of 
%  Press, W.H.,  Flannery, B.P., Teukolsky, S.A., Vetterling, W.T.
%  "Numerical Recipes", Cambridge University Pres, Cambridge, 1986, p. 284-286.
%  
%  Fortran translated to Matlab by David Keffer
%  Department of Chemical Engineering, University of Tennessee
%  May,1999
%
%  The code execute by typing 
%
%  brent(ax,bx,cx,tol,iprint,key)
%
%  where
%  ax , bx , and cx are x coordinates such that ax < bx < cx or ax > bx > cx
%  and f(bx) < f(ax) and f(bx) < f(cx)
%  tol is the tolerance
%  f is the function value at the minimum
%  iprint is 1 if you want intermediate values printed
%  key is either 'root' for locating roots or 'min' for locating minima.
%
%  the function must be located in syseqninput.m
%

%
%  check input for consistency
%
fa = fnctn_filter(ax,key);
fb = fnctn_filter(bx,key);
fc = fnctn_filter(cx,key);
if ( ax >= bx & cx >= bx ) 
   fprintf(1,'bracketing inputs are wrong:  bx is less than both ax and cx. \n');
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter(ax,key);
	fb = fnctn_filter(bx,key);
	fc = fnctn_filter(cx,key);
elseif( ax <= bx & cx <= bx )
   fprintf(1,'bracketing inputs are wrong:  bx is greater than both ax and cx. \n');
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter(ax,key);
	fb = fnctn_filter(bx,key);
	fc = fnctn_filter(cx,key);
end
if (fb > fa) 
   fprintf(1,'bracketing inputs are wrong:  f(bx) > f(ax) %9.2e > %9.2e \n', fb, fa);
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter(ax,key);
	fb = fnctn_filter(bx,key);
	fc = fnctn_filter(cx,key);
end
if (fb > fc) 
   fprintf(1,'bracketing inputs are wrong:  f(bx) > f(cx) %9.2e > %9.2e \n', fb, fc);
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter(ax,key);
	fb = fnctn_filter(bx,key);
	fc = fnctn_filter(cx,key);
end

% 
%  parameters
%
%  maximum iterations
maxit = 1000;
% golden section search constants
cgold = 0.3819660;
% small number for zero-root protection
zeps = 1.0e-10;

%
%  define Brent's six abscissas
%
%  a = lower bound on the optimum
%  b = upper bound on the optimum
%  x = best guess to date
%  w = second best guess to date
%  v = previous value of w
%  u = point of most recent function evaluation
%  got it?
%
a = min(ax,cx);
b = max(ax,cx);
v  = bx;
w = v;
x = v;
% e is distance moved on step before last
e = 0.0;
d = 0.0;		%  correction: i had to add this line myself
fx = fb;
fv = fx;
fw = fx;
iter = 0;
%
%   initialize convergence criteria
%
xm = 0.5*(a+b);
tol1 = tol*abs(x)+zeps;
tol2 = 2*tol1;
err = abs(x - xm);
tole = ( tol2 - 0.5*(b-a) );
%
%  start the main loop
%
while (err > tole)
   %
   %  First we need to find a new step-size, d
   %  if the previous step-size was too big, find a new step-size,  d
   %
   if (abs(e) > tol1)
      %
      % construct a trial parabolic fit
      %
      r = (x-w)*(fx-fv);
      q = (x-v)*(fx-fw);
      p = (x-v)*q - (x-w)*r;
      q = q*(q-r);
      if (q > 0) 
         p = -p;
      end
      q = abs(q);
      etemp = e;
      e = d;
      %
      %  test the parabolic fit
      %
      if ( ( abs(p) >= abs(0.5*q*etemp) ) | ( p <= q*(a-x) ) |  ( p >= q*(b-x) ) )            
         %
         % fit is bad
         %
         if (x >= xm)
            e = a - x;
         else
            e = b - x;
         end
         d = cgold*e;
         %fprintf (1, 'fit is bad, d = %9.2e \n ', d);
		else
         %
         % fit is good
         %
         d = p/q;
         u = x+d;
         if ( (u - a < tol2) | (b - u < tol2) )
            %d = sign(tol1, xm-x);  % FORTRAN sign function is different in MATLAB
            d = sign(tol1*(xm-x))*abs(xm-x);
         end         
      	%fprintf (1, 'fit is good, d = %9.2e \n ', d);
      end
	%
   %  if the previous step-size was not too big, find a new step-size,  d
   %
	else
      if (x >= xm)
      	e = a - x;
      else
         e = b - x;
      end
      d = cgold*e;
   end
   
   %
   %  find new abscissa and ordinate, u and fu
   %
   if (abs(d) > tol1)
      u = x+d;
   else
      % u = x+sign(tol1,d); % FORTRAN sign function is different in MATLAB
      u = x + sign(tol1*d)*abs(d);
   end
   fu = fnctn_filter(u,key);
   
   %
   %  decide what to do with our new abscissa and ordinate
   %
   if (fu <= fx )
      if (u >= x)
         a = x;
      else
         b = x;
      end
      v = w;
      fv = fw;
      w = x;
      fw = fx;
      x = u;
      fx = fu;
   else
      if (u < x)
         a = u;
      else
         b = u;
      end
      if ( fu <= fw | w == x )
         v  = w;
         fv = fw;
         w = u;
         fw = fu;
      elseif ( fu <= fv | v == x | v == w )
         v = u;
         fv = fu;
      end
   end

   %
   %  check for convergence
   %
   xm = 0.5*(a+b);
   tol1 = tol*abs(x)+zeps;
   tol2 = 2*tol1;
   err = abs(x - xm);
   tole = tol2 - 0.5*(b-a) ;  
   %
   %  check iterations
   %
   iter = iter +1;
end
f = fx;
xmin = x;
%
%if (iprint == 1)
%      fprintf (1, 'boundary 1:  a = %9.2e f(a) = % 9.2e \n', a, fnctn_filter(a,key) );
%      fprintf (1, 'boundary 2:  b = %9.2e f(b) = % 9.2e \n', b, fnctn_filter(b,key) );
%      fprintf (1, 'best guess:  x = %9.2e f(x) = % 9.2e \n', x, fx );
%      fprintf (1, '2nd best guess:  w = %9.2e f(w) = % 9.2e \n', w, fw );
%      fprintf (1, 'previous w:  v = %9.2e f(v) = % 9.2e \n', v, fv );
%      fprintf (1, 'most recent point:  u = %9.2e f(u) = % 9.2e \n', u, fu );
%      fprintf (1, 'error = %9.2e iter = %5i \n', err, iter );
%      fprintf (1, 'ANSWER = %13.6e \n', x);
%end



function f = fnctn_filter(x,key);
global pcom xcom ncom
for j = 1:1:ncom
   xt(j) = pcom(j) + x*xcom(j);   
end
f = funkeval(xt);
