function [f,xmin] = dbrentn(ax,bx,cx,tol,iprint,key)
%
%  This routine is called the  Brent Method"
%  for finding a minimum in n dimensions.
%
%  THIS CODE ONLY FINDS MINIMA, NOT MAXIMA.
%
%  This code is based on the "brent" code of 
%  Press, W.H.,  Flannery, B.P., Teukolsky, S.A., Vetterling, W.T.
%  "Numerical Recipes", Cambridge University Pres, Cambridge, 1986, p. 284-286.
%  
%  Fortran translated to Matlab by David Keffer
%  Department of Chemical Engineering, University of Tennessee
%  May,1999
%
%  The code execute by typing 
%
%  dbrentn(ax,bx,cx,tol,iprint,key)
%
%  where
%  ax , bx , and cx are x coordinates such that ax < bx < cx or ax > bx > cx
%  and f(bx) < f(ax) and f(bx) < f(cx)
%  tol is the tolerance
%  f is the function value at the minimum
%  iprint is 1 if you want intermediate values printed
%  key is either 'root' for locating roots or 'min' for locating minima.
%
%  the function must be located in syseqninput.m
%

%
%  check input for consistency
%
fa = fnctn_filter_dbrentn(ax,key);
fb = fnctn_filter_dbrentn(bx,key);
fc = fnctn_filter_dbrentn(cx,key);
if ( ax >= bx & cx >= bx ) 
   fprintf(1,'bracketing inputs are wrong:  bx is less than both ax and cx. \n');
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter_dbrentn(ax,key);
	fb = fnctn_filter_dbrentn(bx,key);
	fc = fnctn_filter_dbrentn(cx,key);
elseif( ax <= bx & cx <= bx )
   fprintf(1,'bracketing inputs are wrong:  bx is greater than both ax and cx. \n');
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter_dbrentn(ax,key);
	fb = fnctn_filter_dbrentn(bx,key);
	fc = fnctn_filter_dbrentn(cx,key);
end
if (fb > fa) 
   fprintf(1,'bracketing inputs are wrong:  f(bx) > f(ax) %9.2e > %9.2e \n', fb, fa);
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter_dbrentn(ax,key);
	fb = fnctn_filter_dbrentn(bx,key);
	fc = fnctn_filter_dbrentn(cx,key);
end
if (fb > fc) 
   fprintf(1,'bracketing inputs are wrong:  f(bx) > f(cx) %9.2e > %9.2e \n', fb, fc);
   [ax, bx, cx] = mnbrak1(ax,bx,iprint,key);
   fprintf (1, 'new brackets: a = %9.2e b = %9.2e c = %9.2e \n', ax, bx, cx);
	fa = fnctn_filter_dbrentn(ax,key);
	fb = fnctn_filter_dbrentn(bx,key);
	fc = fnctn_filter_dbrentn(cx,key);
end

% 
%  parameters
%
%  maximum iterations
maxit = 1000;
% golden section search constants
cgold = 0.3819660;
% small number for zero-root protection
zeps = 1.0e-10;

%
%  define Brent's six abscissas
%
%  a = lower bound on the optimum
%  b = upper bound on the optimum
%  x = best guess to date
%  w = second best guess to date
%  v = previous value of w
%  u = point of most recent function evaluation
%  got it?
%
a = min(ax,cx);
b = max(ax,cx);
v  = bx;
w = v;
x = v;
% e is distance moved on step before last
e = 0.0;
d = 0.0;		%  correction: i had to add this line myself
fx = fb;
fv = fx;
fw = fx;
iter = 0;
%  compute the derivatives numerically
dx = funkd_translator(x,key);
dv = dx;
dw = dx;
ok1 = 0;
ok2 = 0;
%
%   initialize convergence criteria
%
xm = 0.5*(a+b);
tol1 = tol*abs(x)+zeps;
tol2 = 2*tol1;
err = abs(x - xm);
tole = ( tol2 - 0.5*(b-a) );
%
%  start the main loop
%
while (err > tole)
   %
   %  First we need to find a new step-size, d
   %  if the previous step-size was too big, find a new step-size,  d
   %
   if (abs(e) > tol1)
      %
      % construct a trial parabolic fit
      %
      d1 = 2*(b-a);
      d2 = d1;
      %
      %  apply secant method
      %
      if ( abs(dx-dw) > 1.e-12 )
         d1 = (w - x)*dx/(dx-dw);
      end
      if ( abs(dx-dv) > 1.e-12 )
         d2 = (v - x)*dx/(dx-dv);
      end
      u1 = x +d1;
      u2 = x+d2;
      %
      %  check and see if either of these 2 estimates are good
      %  good means ok1 = 1 or ok2 = 1
      %
      if ( ( (a-u1)*(u1-b) > 0 ) & ( dx*d1 <= 0 ) )
         ok1 = 1;
      else
         ok1 = 0;
      end
      if ( ( (a-u2)*(u2-b) > 0 ) & ( dx*d2 <= 0 ) )
         ok2 = 1;
      else
         ok2 = 0;
      end
      olde = e;
      e = d;
      % both derivative estimates are bad
      if (ok1 == 0 & ok2 == 0)
         if (dx >= 0.0)
            e = a -x;
         else
            e = b - x;
         end
         d = 0.5*e; % bisection not golden rule
      else
      	% both derivative estimates are good, take smaller one
      	if (ok1 == 1 & ok2 == 1)
         	if ( abs(d1) < abs(d2) )
            	d = d1;
         	else
            	d = d2;
         	end
      	% derivative estimate 1 is good, use it
      	elseif (ok1 == 1)
         	d = d1;
      	% derivative estimate 2 is good, use it
      	else
         	d = d2;
      	end
      	if ( abs(d) > abs(0.5*olde) )
         	if (dx >= 0.0)
            	e = a -x;
         	else
            	e = b - x;
         	end
         	d = 0.5*e; % bisection not golden rule
      	else
         	u = x+d;
         	if ( (u - a < tol2 ) | (b-u) < tol2 ) 
            	d = sign(tol1*(xm-x))*abs(xm-x);
         	end
         end
      end
	%
   %  if the previous step-size was not too big, find a new step-size,  d
   %
	else
      if (x >= xm)
      	e = a - x;
      else
         e = b - x;
      end
      d = 0.5*e;  % bisection not golden rule
   end
   
   %
   %  find new abscissa and ordinate, u and fu
   %
   if (abs(d) > tol1)
      u = x+d;
      fu = fnctn_filter_dbrentn(u,key);
      du = funkd_translator(u,key);
      [a,b,v,w,x,fv,fw,fx,dv,dw,dx] = bookkeep(a,b,v,w,x,fv,fw,fx,dv,dw,dx,u,fu,du);
   else
      % u = x+sign(tol1,d); % FORTRAN sign function is different in MATLAB
      u = x + sign(tol1*d)*abs(d);
      fu = fnctn_filter_dbrentn(u,key);
      if ( fu <= fx) 
         du = funkd_translator(u,key);
         [a,b,v,w,x,fv,fw,fx,dv,dw,dx] = bookkeep(a,b,v,w,x,fv,fw,fx,dv,dw,dx,u,fu,du);
      else
         %  your smallest step takes you up hill, you are done
         f = fx;
			xmin = x;
         return
      end
   end

   %
   %  check for convergence
   %
   xm = 0.5*(a+b);
   tol1 = tol*abs(x)+zeps;
   tol2 = 2*tol1;
   err = abs(x - xm);
   tole = tol2 - 0.5*(b-a) ;  
   %
   %  check iterations
   %
   iter = iter +1;
   if (iter > maxit)
      fprintf (1, 'boundary 1:  a = %9.2e f(a) = % 9.2e \n', a, fnctn_filter_dbrentn(a,key) );
      fprintf (1, 'boundary 2:  b = %9.2e f(b) = % 9.2e \n', b, fnctn_filter_dbrentn(b,key) );
      fprintf (1, 'best guess:  x = %9.2e f(x) = % 9.2e \n', x, fx );
      fprintf (1, '2nd best guess:  w = %9.2e f(w) = % 9.2e \n', w, fw );
      fprintf (1, 'previous w:  v = %9.2e f(v) = % 9.2e \n', v, fv );
      fprintf (1, 'most recent point:  u = %9.2e f(u) = % 9.2e \n', u, fu );
      fprintf (1, 'error = %9.2e iter = %5i \n', err, iter );
      error('maximum number of iterations exceeded ');
   end

end
f = fx;
xmin = x;
%
if (iprint == 1)
      fprintf (1, 'boundary 1:  a = %9.2e f(a) = % 9.2e \n', a, fnctn_filter_dbrentn(a,key) );
      fprintf (1, 'boundary 2:  b = %9.2e f(b) = % 9.2e \n', b, fnctn_filter_dbrentn(b,key) );
      fprintf (1, 'best guess:  x = %9.2e f(x) = % 9.2e \n', x, fx );
      fprintf (1, '2nd best guess:  w = %9.2e f(w) = % 9.2e \n', w, fw );
      fprintf (1, 'previous w:  v = %9.2e f(v) = % 9.2e \n', v, fv );
      fprintf (1, 'most recent point:  u = %9.2e f(u) = % 9.2e \n', u, fu );
      fprintf (1, 'error = %9.2e iter = %5i \n \n', err, iter );
      fprintf (1, 'ANSWER = %13.6e \n', x);
end





function [a,b,v,w,x,fv,fw,fx,dv,dw,dx] = bookkeep(a,b,v,w,x,fv,fw,fx,dv,dw,dx,u,fu,du)
   	%
   	%  decide what to do with our new abscissa and ordinate
      %
      if (fu <= fx )
      	if (u >= x)
         	a = x;
      	else
         	b = x;
      	end
      	v = w;
      	fv = fw;
      	dv = dw;
      	w = x;
      	fw = fx;
      	dw = dx;
      	x = u;
      	fx = fu;
      	dx = du;
   	else
      	if (u < x)
         	a = u;
      	else
         	b = u;
      	end
      	if ( fu <= fw | w == x )
         	v  = w;
         	fv = fw;
         	dv = dw;
         	w = u;
         	fw = fu;
         	dw = du;
      	elseif ( fu <= fv | v == x | v == w )
         	v = u;
         	fv = fu;
         	dv = du;
      	end
      end

%
%  numerically estimate derivative
%
function df = funkd_translator(x,key)
global pcom xcom ncom
%
% convert 1-D x to n-D line along xcom
%
for j = 1:1:ncom
   xt(j) = pcom(j) + x*xcom(j);   
end
%
%  evaluate n partial derivatives
%
for i = 1:1:ncom
   dfi(i) = funkdii(xt,key,i);
end
%
%  get the total derivative
%
df = 0.0;
for i = 1:1:ncom
   df = df + dfi(i)*xcom(i);
end

function df = funkdii(x,key,idim)
dxcon = 0.01;
ddx = min(dxcon*x(idim),dxcon);
i12dx = 1.0/(12.0*ddx);
xp(1) = x(idim) - 2*ddx;
xp(2) = x(idim) - ddx;
xp(3) = x(idim) + ddx;
xp(4) = x(idim) + 2*ddx;
for j = 1:1:4
   x(idim) = xp(j);
	fp(j) = fnctn_filter(x,key);
end
df = i12dx*( -fp(4) + 8*fp(3) - 8*fp(2) + fp(1) );



%
%  evaluate the function given the full vector from funkdii
%
function y = fnctn_filter(x,key);
f = syseqninput(x);
fsize=max(size(f));
y = sqrt(sum(f.*f)/fsize);


%
%  evaluate the function given the single variable from dbrentn
%
function f = fnctn_filter_dbrentn(x,key);
global pcom xcom ncom
for j = 1:1:ncom
   xt(j) = pcom(j) + x*xcom(j);   
end
f = funkeval(xt);
