function [x,err,f] = nrnd_optn(x0,tol,iprint)
%
%  [x,err,f] = nrnd_optn(x0,tol,iprint)
%
% This script will perform optimization by the
% Newton Raphson with Numerical Derivatives method
% for a system of nonlinear algebraic equations.
%
% inputs:
% x0 = initial guess of x, column vector of length n
% tol = RMS tolerance on relative error on x, scalar
% iprint = value of 1 requests iteration information
% f(x) entered in the function "funkeval" in file funkeval.m
%
% outputs:
% 
% x = converged solution, column vector of length n
% f = RMS value of f, scalar
% err = RMS value of relative error on x, scalar
%
% example usage:
%
%  [x,err,f] = nrnd_optn([4,8],1.0e-6,1)
%
% Author:  David Keffer 
% original date: May, 1999
% last modified date: April 4, 2013
%

%
%  maximum iterations
%
maxit = 1000;
%
%  system size
%
n = max(size(x0));
%
%  allocate variables
%
Residual = zeros(n,1); 
Jacobian = zeros(n,n);
InvJ = zeros(n,n);
dx = zeros(n,1);
x = zeros(n,1);
xold = zeros(n,1);
%
% extra variables for numerical derivatives
%
dxcon = zeros(n,1);
%
%  initialize the size of the discretization 
% for numerical differentiation
%
dxcon(1:n) = 0.01;
%
%  initialization
%
x = x0;
err = 100.0;
iter = 0;
%
% convergence loop
%
while ( err > tol ) 
   for j = 1:1:n
   	dx(j) = min(dxcon(j)*x(j),dxcon(j));
   end
   %
   %  evaluate residual and Jacobian
   %
   Residual = dfunkeval(x,dx,n);
   Jacobian = d2funkeval(x,dx,n);
   %
   %  evaluate new estimate of solution
   %
   xold = x;
   invJ = inv(Jacobian);
   deltax = -invJ*Residual;
   for j = 1:1:n
      x(j) = xold(j) + deltax(j);
   end
   %
   %  check for convergence
   %
   iter = iter +1;
   err = sqrt( sum(deltax.^2) /n  ) ;
   f = sqrt(sum(Residual.*Residual)/n);
   if (iprint == 1)
      fprintf (1,'iter = %4i, err = %9.2e f = %9.2e \n ', iter, err, f);
   end
   if ( iter > maxit)
      Residual
      error ('maximum number of iterations exceeded');
   end
end


%
%  this routine calculates n partial first derivatives
%
function df = dfunkeval(x,dx,n)
df = zeros(n,1);
for i = 1:1:n 
   xtemp(1:n) = x(1:n);
   xtemp(i) = x(i) + dx(i);
   fp = funkeval(xtemp);
   xtemp(i) = x(i) - dx(i);
   fn = funkeval(xtemp);
   df(i) = (fp - fn)/(2*dx(i));
end

%
%  this routine calculates the n^2 numerical partial second derivatives
%
function Jacobian = d2funkeval(x,dx,n)
Jacobian = zeros(n,n);
%
%  diagonal elements
%
for i = 1:1:n
   xtemp(1:n) = x(1:n);
   xtemp(i) = x(i) + dx(i);
   fp = funkeval(xtemp);
   xtemp(i) = x(i);
   fo = funkeval(xtemp);
   xtemp(i) = x(i) - dx(i);
   fn = funkeval(xtemp);
   Jacobian(i,i) = (fp - 2*fo + fn)/(dx(i)*dx(i));
end
%
%  upper-triangular diagonal elements
%
for i = 1:1:n-1
   for j = i+1:1:n
      xtemp(1:n) = x(1:n);
      xtemp(i) = x(i) + dx(i);
      xtemp(j) = x(j) + dx(i);
      fpp = funkeval(xtemp);
      xtemp(i) = x(i) + dx(i);
      xtemp(j) = x(j) - dx(i);
      fpn = funkeval(xtemp);
      xtemp(i) = x(i) - dx(i);
      xtemp(j) = x(j) + dx(i);
      fnp = funkeval(xtemp);
      xtemp(i) = x(i) - dx(i);
      xtemp(j) = x(j) - dx(i);
      fnn = funkeval(xtemp);
      Jacobian(i,j) = (fpp - fpn -fnp + fnn)/(4*dx(i)*dx(j));
   end   
end
%
%  lower triangular diagonal elements by symmetry
%
for i = 2:1:n
   for j = 1:1:i-1
      Jacobian(i,j) = Jacobian(j,i);
   end
end

